;******************************************************************************
;*                                                                            *
;*   Guillermito Cube Mod v4 - July 2006 - http://www.guillermito2.net        *
;*        [copyleft and open source]        guillermito@gmail.com             *
;*                                                                            *
;*                                                                            *
;*                                                                            *
;* What is it ?                                                               *
;* ------------                                                               *
;*                                                                            *
;* A small utility to interactively change the main graphics of the Mobiblu   *
;* DAH-1500i cube MP3 player (boot logo, 6 frames animation at startup, and   *
;* 8 frames animation when you quit). It supports recent firmware versions :  *
;* 100.100.105, 100.100.121 and 100.100.122.                                  *
;* If they upgrade the firmware, my program will probably need some tweaking, *
;* because I use some hardcoded offsets. Send me an email if it happens.      *
;*                                                                            *
;*                                                                            *
;* How to use it ?                                                            *
;* ---------------                                                            *
;*                                                                            *
;* Before :                                                                   *
;*                                                                            *
;* First prepare whatever serie of images that you will use, with your        *
;* favorite image editor (The Gimp, IrfanView, Photoshop). I included some    *
;* examples in this archive. They have to :                                   *
;*                                                                            *
;*  - be uncompressed Windows Bitmaps (.bmp).                                 *
;*  - have a width of 64 an a height of 48 pixels.                            *
;*  - be in 24 bits per pixel color format, aka "16.7 millions of colors",    *
;*    which is very often the default .bmp format for color images.           *
;*  - use only black or white pixels. It is not an absolute obligation, but   *
;*    if they are not, my program will decide which ones should be black and  *
;*    which ones should be white, and the result will be ugly.                *
;*                                                                            *
;* Once you have your images, you just follow the 4 steps, as described :     *
;*                                                                            *
;*  1. Select the Mobiblu firmware "resource.bin", where all the graphics     *
;*     are stored. It would be a good idea to make a copy of the original     *
;*     one before anything. In any case you have a copy on the installation   *
;*     CD, and also you can download it from the Mobiblu website. This file   *
;*     is present by default in the directory : C:\Program Files\MP3 Update   *
;*     (1 Gb Cube) or C:\Program Files\Audio Player (2 Gb Cube).              *
;*                                                                            *
;*  2. You will now see the graphics present inside the selected firmware.    *
;*     If you don't see anything or mangled images, stop here and send me an  *
;*     e-mail. Now you just have to click on any of those 16 frames to        *
;*     replace it by a .bmp of your choice that you will select.              *
;*                                                                            *
;*  3. This will overwrite your old firmware by a new copy that contains      *
;*     your images. You can keep the program open at this point and           *
;*     continue your modifications later.                                     *
;*                                                                            *
;*  4. Quit.                                                                  *
;*                                                                            *
;* After :                                                                    *
;*                                                                            *
;* Once the new "resource.bin" is written on the disk, you can update         *
;* the firmware manually with the Mobiblu tool. Don't forget to uncheck       *
;* the "Format Data Area" option, or the mp3s present on the Cube will be     *
;* deleted.                                                                   * 
;*                                                                            *
;* Maybe if I have some time I will update this tool to change some other     *
;* things. Have fun, and happy hacking.                                       *
;*                                                                            *
;* As always, you use this tool on your own responsability. I tested the      *
;* precedent versions with my Cubes and they worked fine. As I have now       *
;* killed my two Mobiblu because of too much software and hardware hacking,   *
;* I couldn't test this version myself on a Cube. My educated guess is that   *
;* there should be no problem, as I just added pointers to the images inside  *
;* the new firmware. But once again, if you decide to use this tool, I am     *
;* not responsible for anything ugly that may happen. Feedback welcome        *
;* at guillermito@gmail.com.                                                  *
;*                                                                            *
;*                                                                            *
;* Compile it yourself:                                                       *
;* --------------------                                                       *
;*                                                                            *
;*          Resource: gorc.exe /r gcubemod.rc                                 *
;*          Assemble: tasm32 /ml /m3 /z /t gcubemod                           *
;*              Link: tlink32 -Tpe -aa gcubemod,,,import32,,gcubemod.res      *
;*          Compress: upx -9 gcubemod.exe                                     *
;*                                                                            *
;*                                                                            *
;* History :                                                                  *
;* ---------                                                                  *
;*                                                                            *
;* Version 4 : Added compatibility with new firmware 100.100.122              *
;* Version 3 : Added compatibility with new firmware 100.100.121              *
;* Version 2 : Now interactive, you can see the images you're changing        *
;*             Added mod of each frame of boot an quit animations             *
;* Version 1 : Change static boot logo only                                   *
;*                                                                            *
;****************************************************************************** 

callW macro x
extrn x:PROC
call x
endm

.486
.model flat

.data

;****************** structures *************************

openfilename_struct:
lStructSize        dd openfilename_struct_size
hwndOwner          dd 0
hInstance          dd 0
lpstrFilter        dd ?
lpstrCustomFilter  dd 0
nMaxCustFilter     dd 0
nFilterIndex       dd 0
lpstrFile          dd ?
nMaxFile           dd 255
lpstrFileTitle     dd 0
nMaxFileTitle      dd 32
lpstrInitialDir    dd 0
lpstrTitle         dd 0
Flags              dd 1000h+4h+200000h
nFileOffset        dw 0
nFileExtension     dw 0
lpstrDefExt        dd 0
lCustData          dd 0
lpfnHook           dd 0
lpTemplateName     dd 0
openfilename_struct_size equ $-offset openfilename_struct
filter_bin         db "Mobiblu firmware file",0,"resource.bin",0
                   db "All files",0,"*.*",0,0
filter_bmp         db "64x48 24-bits Bitmap file",0,"*.bmp",0
                   db "All files",0,"*.*",0,0
namebuffer_bin     db 255 dup(0)
namebuffer_bmp     db 255 dup(0)

bitmapinfo:
biSize             dd bmpinfosize ;biSize
biWidth            dd 64          ;biWidth
biHeight           dd 48          ;biHeight
biPlanes           dw 1           ;biPlanes
biBitCount         dw 24          ;biBitCount
biCompression      dd 0           ;biCompression
biSizeImage        dd 0           ;biSizeImage
biXPelsPerMeter    dd 0           ;biXPelsPerMeter
biYPelsPerMeter    dd 0           ;biYPelsPerMeter
biClrUsed          dd 0           ;biClrUsed
biClrImportant     dd 0           ;biClrImportant
bmpinfosize        equ $-offset bitmapinfo

;****************** other stuff *************************

modhandle          dd ?
dialog_name        db "dialogue",0
hWnd               dd ?
uMsg               dd ?
wParam             dd ?
file_handle        dd ?
file_size          dd ?
fmw_file_size      dd ?
fmw_mem_buffer     dd ?
bmp_mem_buffer     dd ?
file_nb_bytes_read dd ?
pixels_data_start  dd ?
encode_mem_buffer  dd ?
encode_size        dd ?
titlebox           db "Guillermito Cube Mod v4",0
prob_firmware      db "Firmware has a bad size or version signature,",0dh,0ah
                   db "or there was a general read/memory problem",0
prob_bitmap        db "Bitmap is not 64x48 pixels or 24 bits per pixel (aka 16.7 millions colors),",0dh,0ah
                   db "or there was a general read/memory problem",0
prob_save          db "There seems to be a general problem saving the new firmware",0
sig_firmware_105   db "Product Version: 100.100.105",0dh,0ah
                   db "Component Version: 002.511.100",0dh,0ah
sig_firmware_121   db "Product Version: 100.100.121",0dh,0ah
                   db "Component Version: 002.600.100",0dh,0ah
sig_firmware_122   db "Product Version: 100.100.122",0dh,0ah
                   db "Component Version: 002.600.100",0dh,0ah
size_sig           equ $-offset sig_firmware_122
hbmp               dd ?
offsetbitmappixels dd ?
the_DC             dd ?
current_image      dd ?
text_to_enable     db 201,202,250,251,252,253,254 

;****************** harcoded images offsets *************************

images_and_dede_offset_table dd ?

images_and_dede_offset_table_105: ;offset image, offset dede, dialog ID
dd 045294h,0      ,300 ;boot logo
dd 30ea7fh,0      ,310 ;anim logo 1
dd 30ec0eh,30ec41h,311 ;anim logo 2
dd 30ed9fh,30ee41h,312 ;anim logo 3
dd 30ef30h,30f041h,313 ;...
dd 30f0c1h,0      ,314
dd 30f252h,0      ,315
dd 3101f6h,310241h,320
dd 310387h,310441h,321 
dd 310518h,310641h,322 
dd 3106a9h,0      ,323 
dd 310838h,310841h,324 
dd 3109c9h,310a41h,325 
dd 310b5ah,310c41h,326 
dd 310cebh,310e41h,327 

images_and_dede_offset_table_121: ;offset image, offset dede, dialog ID
dd 04aa29h,04aa41h,300 ;boot logo
dd 2c20d9h,2c2241h,310 ;anim logo 1
dd 2c226ah,0      ,311 ;anim logo 2
dd 2c23f9h,2c2441h,312 ;anim logo 3
dd 2c258ah,2c2641h,313 ;anim logo 4
dd 2c271bh,2c2841h,314 ;anim logo 5
dd 2c28ach,0      ,315 ;anim logo 6
dd 2c3852h,0      ,320 ;anim quit 1
dd 2c39e1h,2c3a41h,321 ;anim quit 2 
dd 2c3b72h,2c3c41h,322 ;anim quit 3
dd 2c3d03h,2c3e41h,323 ;anim quit 4 
dd 2c3e94h,0      ,324 ;anim quit 5 
dd 2c4023h,2c4041h,325 ;anim quit 6 
dd 2c41b4h,2c4241h,326 ;anim quit 7 
dd 2c4345h,2c4441h,327 ;anim quit 8 

images_and_dede_offset_table_122: ;offset image, offset dede, dialog ID
dd 04aa5bh,0      ,300 ;boot logo
dd 2c20fdh,2c2241h,310 ;anim logo 1
dd 2c228eh,0      ,311 ;anim logo 2
dd 2c241dh,2c2441h,312 ;anim logo 3
dd 2c25aeh,2c2641h,313 ;anim logo 4
dd 2c273fh,2c2841h,314 ;anim logo 5
dd 2c28d0h,2c2a41h,315 ;anim logo 6
dd 2c3876h,0      ,320 ;anim quit 1
dd 2c3a05h,2c3a41h,321 ;anim quit 2 
dd 2c3b96h,2c3c41h,322 ;anim quit 3
dd 2c3d27h,2c3e41h,323 ;anim quit 4 
dd 2c3eb8h,0      ,324 ;anim quit 5 
dd 2c4049h,0      ,325 ;anim quit 6 
dd 2c41d8h,2c4241h,326 ;anim quit 7 
dd 2c4369h,2c4441h,327 ;anim quit 8 

.code

CODE_POETRY:

;************************************************************************
;*********************  MAIN ROUTINE  ***********************************
;************************************************************************

;--------- get handle ---------------------

push 0
callW GetModuleHandleA
mov modhandle, eax

;--------- show dialog box --------------

push 0
push offset dialog_proc
push 0
push offset dialog_name
push modhandle
callW DialogBoxParamA

;---------- terminate ------------

push -1
callW ExitProcess 

;************************************************************************
;*********************  DIALOG PROCEDURE  *******************************
;************************************************************************

dialog_proc:
pusha
push ebp
mov ebp, esp
mov eax, [ebp+8+20h]    ;hWnd
mov hWnd, eax
mov eax, [ebp+12+20h]   ;uMsg
mov uMsg, eax
mov eax, [esp+16+20h]   ;wParam
mov wParam, eax

;-------------- if the dialog is closed --------------

cmp uMsg, 10h              ;WM_CLOSE
jne no_end
push 0
push hWnd
callW EndDialog
jmp end_dlgproc_return_1

;--------------- if the dialog is started --------------

no_end:
cmp uMsg, 110h             ;WM_INITDIALOG
jne no_start
jmp end_dlgproc_return_1  

;-------------- if there is a command sent to the dialog --------------

no_start:
cmp uMsg, 111h             ;WM_COMMAND  
je there_is_a_command
jmp end_dlgproc_return_0  
there_is_a_command:

;************************************************************************
;*********************  1. FIND FIRMWARE  *******************************
;************************************************************************

cmp wParam, 200            ;button no1
je find_firmware
jmp check_button2
find_firmware:

;-------- choose a file -------------

mov lpstrFilter, offset filter_bin
mov lpstrFile, offset namebuffer_bin
push offset openfilename_struct
callW GetOpenFileNameA
test eax, eax
jnz choose_ok
jmp firmware_problem
choose_ok:

;-------- open it --------------

push 0
push 80h                          ;FILE_ATTRIBUTE_NORMAL
push 3                            ;OPEN_EXISTING
push 0
push 0
push 80000000h+40000000h          ;GENERIC_READ+GENERIC_WRITE
push dword ptr [lpstrFile]
callW CreateFileA                 ;open file
inc eax
jnz open_ok
jmp firmware_problem
open_ok:
dec eax
mov file_handle, eax

;---------- get its size ---------

push 0
push dword ptr file_handle
callW GetFileSize                 ;get file size in bytes
inc eax
jnz size_ok
jmp firmware_problem
size_ok:
dec eax
mov fmw_file_size, eax

;--------- alloc memory for file --------

mov eax, fmw_file_size
add eax, 1024
push eax                         ;number of bytes to allocate
push 40h                         ;40h=LMEM_ZEROINIT
callW LocalAlloc
test eax, eax
jnz alloc_ok
jmp firmware_problem
alloc_ok:
mov fmw_mem_buffer, eax

;------- read entire file ------------

push 0
push offset file_nb_bytes_read
push fmw_file_size
push fmw_mem_buffer
push file_handle
callW ReadFile
test eax, eax
jnz read_ok
jmp firmware_problem
read_ok:
mov eax, file_nb_bytes_read
cmp eax, fmw_file_size
jz size_read_OK
jmp firmware_problem
size_read_OK:

;-------- check if size is that of 100.100.105 -----------

cmp fmw_file_size, 3294102
jz size_OK

;-------- check if size is that of 100.100.121 -----------

cmp fmw_file_size, 3084080
jz size_OK

;-------- check if size is that of 100.100.122 -----------

cmp fmw_file_size, 3084116
jz size_OK
jmp firmware_problem
size_OK:

;------- close it ----------------

push file_handle
callW CloseHandle
test eax, eax
jnz close_ok
jmp firmware_problem
close_ok:

;-------- check 100.100.105 signature ---------

mov esi, fmw_mem_buffer
mov edi, offset sig_firmware_105
mov ecx, size_sig
repz cmpsb
jz sig_OK

;-------- check 100.100.121 signature ---------

mov esi, fmw_mem_buffer
mov edi, offset sig_firmware_121
mov ecx, size_sig
repz cmpsb
jz sig_OK

;-------- check 100.100.122 signature ---------

mov esi, fmw_mem_buffer
mov edi, offset sig_firmware_122
mov ecx, size_sig
repz cmpsb
jz sig_OK
jmp firmware_problem
sig_OK:

;------- show all images inside buttons ----------
 
push hWnd
callW GetDC
mov the_DC, eax

mov ecx, 1+6+8

;------- images are at different places in different firmwares ---------

f105:
cmp fmw_file_size, 3294102
jne f121
mov eax, offset images_and_dede_offset_table_105
mov images_and_dede_offset_table, eax
mov esi, eax
jmp get_all_images
f121:
cmp fmw_file_size, 3084080
jne f122
mov eax, offset images_and_dede_offset_table_121
mov images_and_dede_offset_table, eax
mov esi, eax
jmp get_all_images
f122:
mov eax, offset images_and_dede_offset_table_122
mov images_and_dede_offset_table, eax
mov esi, eax

get_all_images:
mov eax, [esi]
mov ebx, [esi+4]
mov edx, [esi+8]
add esi, 12
call nokia2bmp2buttons
loop get_all_images

;---------- enable next steps and text  -------------

mov esi, offset text_to_enable
mov ecx, 7
xor eax, eax
enable_text:
lodsb
pusha
push eax
push hWnd
callW GetDlgItem
push 1 
push eax
callW EnableWindow
popa
loop enable_text

jmp end_dlgproc_return_0

;-------- firmware problem -----------

firmware_problem:
push 0
push offset titlebox
push offset prob_firmware
push hWnd
callW MessageBoxA

jmp end_dlgproc_return_0

;************************************************************************
;*********************  2. FIND BITMAPS  ********************************
;************************************************************************

;-------- check if an image has been clicked and which one -------------

check_button2:
mov esi, images_and_dede_offset_table
mov ecx, 1+6+8
which_image:
mov eax, [esi+8]
cmp wParam, eax
je image_found
add esi, 12
loop which_image
jmp check_button3

image_found:
mov current_image, esi

;-------- choose a file -------------

mov lpstrFilter, offset filter_bmp
mov lpstrFile, offset namebuffer_bmp
push offset openfilename_struct
callW GetOpenFileNameA
test eax, eax
jnz choose_ok2
jmp bmp_problem
choose_ok2:

;-------- open it --------------

push 0
push 80h                          ;FILE_ATTRIBUTE_NORMAL
push 3                            ;OPEN_EXISTING
push 0
push 0
push 80000000h+40000000h          ;GENERIC_READ+GENERIC_WRITE
push dword ptr [lpstrFile]
callW CreateFileA                 ;open file
inc eax
jnz open_ok2
jmp bmp_problem
open_ok2:
dec eax
mov file_handle, eax

;---------- get its size ---------

push 0
push dword ptr file_handle
callW GetFileSize                 ;get file size in bytes
inc eax
jnz size_ok2
jmp bmp_problem
size_ok2:
dec eax
mov file_size, eax

;--------- alloc memory for file --------

mov eax, file_size
add eax, 1024                    ;add some room for encoding
push eax                         ;number of bytes to allocate
push 40h                         ;40h=LMEM_ZEROINIT
callW LocalAlloc
test eax, eax
jnz alloc_ok2
jmp bmp_problem
alloc_ok2:
mov bmp_mem_buffer, eax

;------- read entire file ------------

push 0
push offset file_nb_bytes_read
push file_size
push bmp_mem_buffer
push file_handle
callW ReadFile
test eax, eax
jnz read_ok2
jmp bmp_problem
read_ok2:
mov eax, file_nb_bytes_read
cmp eax, file_size
jz size_read_OK2
jmp bmp_problem
size_read_OK2:

;------- close it ----------------

push file_handle
callW CloseHandle
test eax, eax
jnz close_ok2
jmp bmp_problem
close_ok2:

;-------- find BMP signature ---------

mov esi, bmp_mem_buffer
lodsw
cmp ax, "MB"
je bm_ok
jmp bmp_problem
bm_ok:

;------- find size and number of bits per pixel ------

add esi, 8
lodsd
add eax, bmp_mem_buffer
mov pixels_data_start, eax
add esi, 4
lodsd
cmp eax, 64d
je x_ok
jmp bmp_problem
x_ok:
lodsd
cmp eax, 48d
je y_ok
jmp bmp_problem
y_ok:
add esi, 2
lodsw
cmp ax, 24d
je this_looks_like_a_good_bmp
jmp bmp_problem

;------ Encode BMP into Nokia/Mobiblu format -----------

this_looks_like_a_good_bmp:
mov eax, bmp_mem_buffer
add eax, file_size
mov encode_mem_buffer, eax

mov ecx, 6
mov esi, pixels_data_start
add esi, 3*64*40
mov edi, encode_mem_buffer

parse_image:
push ecx
mov ecx, 64
push esi

parse_one_block_of_8_lines:
push ecx
xor eax, eax
xor edx, edx
mov ecx, 8

parse_one_column:
rol dl, 1
mov ebx, 8
sub ebx, ecx
mov eax, 64*3
push edx
mul ebx
pop edx
cmp byte ptr [esi+eax], 127
jb black
inc dl
black:
loop parse_one_column

xchg eax, edx
stosb
add esi, 3
pop ecx
loop parse_one_block_of_8_lines

pop esi
sub esi, 64*3*8 
pop ecx
loop parse_image

finish_encoding:
mov eax, edi
sub eax, encode_mem_buffer
mov encode_size, eax

;------- check if it's 384 bytes long ----------

cmp encode_size, 384
jnz bmp_problem 

;----------- get offsets in the table ------------

mov esi, current_image
lodsd
push eax
mov edi, fmw_mem_buffer
add edi, eax
lodsd
push eax
mov ebx, fmw_mem_buffer
add ebx, eax
lodsd
mov edx, eax

;----------- replace boot logo in memory ------------

mov esi, encode_mem_buffer
mov ecx, 384
replace_and_check_DEDE:
mov al, [edi]
cmp al, 0deh
jne no_DEDE2
cmp byte ptr [edi+1], 0deh
jne no_DEDE2
cmp edi, ebx
jne no_DEDE2
add edi,2 
no_DEDE2:
movsb
loop replace_and_check_DEDE

;--------- show new image in the button -----------

pop ebx
pop eax
call nokia2bmp2buttons

jmp end_dlgproc_return_0

;-------- bmp problem -----------

bmp_problem:
push 0
push offset titlebox
push offset prob_bitmap
push hWnd
callW MessageBoxA

jmp end_dlgproc_return_0

;************************************************************************
;******************  3. SAVE NEW FIRMWARE  ******************************
;************************************************************************

check_button3:
cmp wParam, 201                    ;button no3
je save_fw
jmp check_button4
save_fw:

;------- open a new file on the disk ----------------

push 0                      ;handle of file with attributes to copy 
push 80h                    ;file attributes (80h=FILE_ATTRIBUTE_NORMAL)
push 2                      ;how to create (2=CREATE_ALWAYS)
push 0                      ;address of security descriptor 
push 0                      ;share mode (0=Prevents the file from being shared)
push 40000000h              ;access (read-write) mode (40000000h=GENERIC_WRITE)
push offset namebuffer_bin  ;address of name of the file
call CreateFileA
inc eax
jnz open_ok3
jmp save_problem
open_ok3:
dec eax
mov file_handle, eax

;------- write buffer in it ----------------

push 0
push offset file_nb_bytes_read          ;address of number of bytes written 
push fmw_file_size                      ;number of bytes to write
push fmw_mem_buffer                     ;address of data to write to file
push file_handle                        ;handle of file to write to
callW WriteFile
test eax, eax
jnz write_ok
jmp save_problem
write_ok:
mov eax, file_nb_bytes_read
cmp eax, fmw_file_size
jz size_write_ok
jmp save_problem
size_write_ok:

;------- close it ----------------

push file_handle
callW CloseHandle
test eax, eax
jnz close_ok3
jmp save_problem
close_ok3:

jmp end_dlgproc_return_0

;-------- save problem -----------

save_problem:
push 0
push offset titlebox
push offset prob_save
push hWnd
callW MessageBoxA

jmp end_dlgproc_return_0

;************************************************************************
;***********************  4. QUIT  **************************************
;************************************************************************

check_button4:
cmp wParam, 202                    ;button no4
je ze_end
jmp end_dlgproc_return_0
ze_end:

;------- free memory ----------------

push fmw_mem_buffer
callW LocalFree
push bmp_mem_buffer
callW LocalFree

;--------- end dialog proc -----------------

push 0
push hWnd
callW EndDialog
jmp end_dlgproc_return_1

end_dlgproc_return_0:
leave
popa
xor eax, eax
ret 16

end_dlgproc_return_1:
leave
popa
xor eax, eax
inc eax
ret 16

;************************************************************************
;*********************  SOME ROUTINES ***********************************
;************************************************************************

nokia2bmp2buttons:

;---------- create bmp in memory ----------------

pusha
push 0
push 0
push offset offsetbitmappixels
push 0                          ;DIB_RGB_COLORS
push offset bitmapinfo
push the_DC
callW CreateDIBSection
mov hbmp, eax
popa

;--------- do the conversion --------------

pusha
mov esi, fmw_mem_buffer
add esi, eax
mov edi, offsetbitmappixels  
add ebx, fmw_mem_buffer
inc ebx
 
add edi, 64*40*3
mov ecx, 6

parse_384_bytes:
push ecx
mov ecx, 64

parse_64_bytes:
push ecx
lodsb
cmp al, 0deh
jne no_DEDE
cmp byte ptr [esi], 0deh
jne no_DEDE
cmp esi, ebx
jne no_DEDE
inc esi
lodsb  
no_DEDE:
mov ecx, 8

parse_1_byte:
push eax
and al, 10000000b
cmp al, 0
je black2
mov al, 220        ;blue
stosb
mov al, 170        ;green
stosb
mov al, 50         ;red
stosb
jmp no_black
black2:
xor eax, eax
stosb
stosb
stosb
no_black:
pop eax
rol al, 1
add edi, (64*3)-3
loop parse_1_byte

pop ecx
sub edi, ((64*3)*8)-3
loop parse_64_bytes

pop ecx
sub edi, ((64*3)*9)
loop parse_384_bytes
 
;--------- show the bmp in a button --------------

push edx
push hWnd
callW GetDlgItem
push eax
	
push hbmp
push 0
push 0f7h            ;BM_SETIMAGE
push eax
callW SendMessageA

pop eax
push 1
push eax
callW EnableWindow
	
popa
ret

end CODE_POETRY
